import xml.etree.cElementTree as ET
import functools
from datetime import date


class Builder(object):

    def __call__(self, tag, *children, **attrib):
        elem = ET.Element(tag, attrib)
        for item in children:
            if isinstance(item, dict):
                elem.attrib.update(item)
            elif ET.iselement(item):
                elem.append(item)
            else:
                if len(elem):
                    elem[-1].tail = (elem[-1].tail or "") + xmli_str(item)
                else:
                    elem.text = (elem.text or "") + xmli_str(item)
        return elem

    def __getattr__(self, tag):
        return functools.partial(self, tag)

E = Builder()


def xmli_str(value):
    if isinstance(value, bool):
        return 'true' if value else 'false'
    elif isinstance(value, date):
        return value.strftime('%Y-%m-%dT00:00:00.000Z')
    else:
        return unicode(value)


def get_node(root, path):
    node = root
    for name in path.split('/'):
        child = node.find(name)
        if child is None:
            child = ET.SubElement(node, name)
        node = child
    return node


def set_value(root, path, value):
    node = get_node(root, path)
    node.text = xmli_str(value)
